import subprocess
import sys
import re

def run_command(command):
    """Run a shell command and return the output"""
    try:
        result = subprocess.run(command, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, universal_newlines=True)
        return result.stdout.strip()
    except subprocess.CalledProcessError as e:
        # Special handling for the dnf update --assumeno command
        if "dnf update --assumeno" in command:
            return e.stdout.strip()  # Return stdout (even if there was an error)
        else:
            # For other commands, keep the existing behavior
            return ""


def check_minimal_install_group():
    """Check if the 'Minimal Install' group is installed"""
    print("\n------------------------------------------------------------- \nChecking for 'Minimal Install' group \n-------------------------------------------------------------\n")
    group_list_output = run_command("dnf grouplist --installed")
    
    if "Minimal Install" in group_list_output:
        print("✔️ 'Minimal Install' group is installed.\n")
        return True
    else:
        print("⚠️ 'Minimal Install' group not found. Checking if mandatory and default packages of 'Minimum Install' group are installed...\n")
        return False

def get_minimal_install_info():
    """Fetch details of the 'Minimal Install' group"""
    minimal_install_info = run_command("dnf groupinfo 'Minimal Install'")
    mandatory_groups = []

    is_mandatory_section = False

    for line in minimal_install_info.splitlines():
        line = line.strip()
        if line.startswith("Mandatory Groups:"):
            is_mandatory_section = True
        elif line.startswith("Optional Groups:") or not line:
            is_mandatory_section = False
        elif is_mandatory_section:
            mandatory_groups.append(line)

    return mandatory_groups

def get_group_packages(group_name):
    """Retrieve mandatory and default packages for a group"""
    group_info = run_command(f"dnf groupinfo '{group_name}'")
    mandatory_packages, default_packages = [], []
    section = None

    for line in group_info.splitlines():
        line = line.strip()
        if line.startswith("Mandatory Packages:"):
            section = "mandatory"
        elif line.startswith("Default Packages:"):
            section = "default"
        elif line.startswith("Optional Packages:") or not line:
            section = None
        elif section == "mandatory":
            mandatory_packages.append(line.split("(")[0].strip())
        elif section == "default":
            default_packages.append(line.split("(")[0].strip())

    return mandatory_packages, default_packages

def check_packages_installed(packages):
    """Verify if specific packages are installed"""
    missing_packages = []
    for package in packages:
        rpm_check = run_command(f"rpm -q {package}")
        if "is not installed" in rpm_check or not rpm_check.strip():
            missing_packages.append(package)
    return missing_packages

def validate_minimal_install():
    """Validate the 'Minimal Install' group by checking mandatory packages"""
    mandatory_groups = get_minimal_install_info()
    if not mandatory_groups:
        print("⚠️ No mandatory groups found in the 'Minimal Install' environment!\n")
        return
    
    missing_packages = []
    for group in mandatory_groups:
        mandatory_packages, _ = get_group_packages(group)
        missing_packages.extend(check_packages_installed(mandatory_packages))

    if not missing_packages:
        print("✔️  All mandatory packages for 'Minimal Install' are installed.\n")
    else:
        print(f"⚠️ If using ULN, the following packages are missing: {', '.join(missing_packages)}\n")

def check_ovirt_engine_host_packages():
    """Check for the presence of the 'ovirt-engine' package"""
    print("\n------------------------------------------------------------- \nChecking 'ovirt-engine' package \n-------------------------------------------------------------\n")
    engine_package = "ovirt-engine"
    host_package = "ovirt-host"
    output = run_command(f"dnf list installed {engine_package} {host_package}")
    if engine_package in output and host_package in output:
        print(f"💀Both '{host_package}' and '{engine_package}' packages are installed. Unsupported Configuration.\n")
        return
    elif engine_package in output:
        print(f"⚠️ '{engine_package}' package is installed. DO NOT configure the host on this server.\n")
        return
    elif host_package in output:
        print(f"⚠️ '{host_package}' package is installed. DO NOT configure the engine on this server.\n")
    else:
        print(f"✔️ '{engine_package}' and '{host_package}' packages are not installed. You can Proceed with the configuration of Engine or Host.\n")

def check_enabled_repos():
    """Validate enabled repositories"""
    print("\n------------------------------------------------------------- \nChecking enabled repositories \n-------------------------------------------------------------\n")
    repo_list_output = run_command("dnf repolist enabled")
    #enabled_repos = [line.split()[0] for line in repo_list_output.splitlines()[2:]]  #<---- Original
    enabled_repos = [line.split()[0] for line in repo_list_output.splitlines()[1:]]  #<---- Added by Rehan
    
    required_repos = [
        "ol8_baseos_latest", "ol8_appstream", "ol8_kvm_appstream",
        "ovirt-4.5", "ovirt-4.5-extra", "ol8_gluster_appstream", "ol8_UEKR7"
    ]
    
    extra_repos = [repo for repo in enabled_repos if repo not in required_repos]
    missing_repos = [repo for repo in required_repos if repo not in enabled_repos]

    if not extra_repos and not missing_repos:
        print("✔️ All required repositories are enabled.\n")
    else:
        if extra_repos:
            print(f"⚠️ Extra repositories are enabled: {', '.join(extra_repos)}\n")
        if missing_repos:
            print(f"⚠️ Missing repositories: {', '.join(missing_repos)}\n")
        

def parse_version(version_string):
    """Parse version string into a tuple of integers (major, minor, patch)."""
    match = re.match(r"(\d+)\.(\d+)\.(\d+)", version_string)
    if match:
        return tuple(map(int, match.groups()))
    return None

def check_ansible_version():
    """Verify Ansible installation and version"""
    print("\n------------------------------------------------------------- \nChecking Ansible version \n-------------------------------------------------------------n")
    output = run_command("ansible --version")
    if not output:
        print("⚠️ Ansible is not installed.\n")
        return
    
    match = re.search(r"ansible \[core (\d+\.\d+\.\d+)\]", output)
    if match:
        version = match.group(1)
        min_version, max_version = (2, 15, 0), (2, 16, 99)
        parsed_version = tuple(map(int, version.split(".")))
        if min_version <= parsed_version <= max_version:
            print(f"✔️ Ansible version {version} is within the supported range.\n")
        else:
            print(f"⚠️ Ansible version {version} is outside the supported range.\n")
    else:
        print("⚠️ Unable to parse Ansible version.\n")


def check_repositories():
    """Validate repository reachability using dnf makecache and dry-run update."""
    print("\n------------------------------------------------------------- \nChecking repository reachability \n-------------------------------------------------------------\n")

    # Check with `dnf makecache`
    print("1: Running `dnf makecache`...\n")
    makecache_output = run_command("dnf makecache")

    if "Metadata cache created" in makecache_output:
        print("✔️ Metadata cache created successfully.\n")
    else:
        print("⚠️ Issues encountered during `dnf makecache`:\n")
        print(makecache_output)

    # Dry-run update with `dnf update --assumeno`
    print("2: Performing dry-run update with `dnf update --assumeno`...\n")
    dry_run_output = run_command("sudo dnf update --assumeno")

    if "Dependencies resolved." in dry_run_output:
        print("✔️ Dry-run update completed successfully. Repositories are reachable.\n")
    else:
        print("⚠️ Errors or warnings encountered during dry-run update:\n")
        print(dry_run_output)
       

def check_firewalld_status():
    """Check if the firewalld service is running."""
    print("\n------------------------------------------------------------- \nChecking firewalld service status \n-------------------------------------------------------------\n")

    try:
        status = run_command("systemctl is-active firewalld")

        if status == "active":
            print("✔️ Firewalld service is running.\n")
            return True
        elif status == "inactive":
            print("⚠️ Firewalld service is not running.\n")
        else:
            print(f"⚠️ Firewalld status: {status}\n")
        return False
    except Exception as e:
        print(f"⚠️ Error checking firewalld status: {e}\n")
        return False

def check_selinux_status():
    """Check and report the current SELinux status."""
    print("\n------------------------------------------------------------- \nChecking SELinux status \n-------------------------------------------------------------\n")

    try:
        sestatus_output = run_command("sestatus")

        if not sestatus_output:
            print("⚠️ SELinux might not be installed or configured.")
            return

        if "enabled" in sestatus_output:
            if "enforcing" in sestatus_output:
                print("✔️ SELinux is enabled and in enforcing mode.\n")
            else:
                print("⚠️ SELinux is enabled but not in enforcing mode.\n")
        else:
            print("⚠️ SELinux is not enabled.\n")

    except Exception as e:
        print(f"⚠️ Error checking SELinux status: {e}\n")



def check_fips_status():
    """Check if FIPS mode is enabled on the system."""
    print("\n------------------------------------------------------------- \nChecking FIPS mode \n-------------------------------------------------------------\n")

    fips_enabled = False

    # Check FIPS via sysctl
    try:
        fips_status = run_command("sysctl -n crypto.fips_enabled")
        if fips_status == "1":
            fips_enabled = True
    except Exception:
        print("⚠️ Error checking FIPS status via sysctl.")

    # Check FIPS via /proc/sys/crypto/fips_enabled
    try:
        with open("/proc/sys/crypto/fips_enabled", "r") as f:
            if f.read().strip() == "1":
                fips_enabled = True
    except FileNotFoundError:
        print("⚠️ /proc/sys/crypto/fips_enabled not found. FIPS check might not apply to your system.")

    # Output FIPS status
    if fips_enabled:
        print("✔️ FIPS mode is enabled.\n")
    else:
        print("✔️ FIPS mode is not enabled.\n")



def main():
    check_minimal_install_group()
    validate_minimal_install()
    check_ovirt_engine_host_packages()
    check_enabled_repos()
    check_ansible_version()
    check_repositories()
    check_firewalld_status()
    check_fips_status()
    check_selinux_status()

if __name__ == "__main__":
    main()
